// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2008  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Apologies for the lack of comments... I never thought this would end up
// online. If you have any questions feel free to email me at:
// ali@arkitus.com
// ------------------------------------------------------------------------------

class Camera
{
  Vector cursorPosition;
  Vector cameraPosition;
  Vector[] nicePoints;
  
  Camera
  ()
  {
    cursorPosition = new Vector( WIDTH/2, HEIGHT/2, 0 );
    cameraPosition = new Vector( 0, 50, SCENE_RADIUS * CAMERA_DISTANCE );
    
    nicePoints = new Vector[4];
    nicePoints[0] = new Vector( 0, -CAMERA_HEIGHT, SCENE_RADIUS * CAMERA_DISTANCE );
    nicePoints[1] = new Vector( SCENE_RADIUS * CAMERA_DISTANCE, -CAMERA_HEIGHT, 0 );
    nicePoints[2] = new Vector( 0, -CAMERA_HEIGHT, -SCENE_RADIUS * CAMERA_DISTANCE );
    nicePoints[3] = new Vector( -SCENE_RADIUS * CAMERA_DISTANCE, -CAMERA_HEIGHT, 0 );
  }
  
  void tick
  ()
  {
    float diffX;
    float diffY;
    
    float rotationX;
    float rotationY;
    
    Vector up;
    Vector cross;

    if ( !mousePressed )
    {
      Vector diff = findClosestNicePoint().sub( cameraPosition );
      
      if ( diff.length2() < SNAP_RADIUS * SNAP_RADIUS )
      {
        cameraPosition = cameraPosition.add( diff.mul( SNAP_SPEED ) );
      }
    }
    
    if ( mousePressed )
    {
      diffX = mouseX - cursorPosition.x;
      diffY = mouseY - cursorPosition.y;
    }
    else
    {
      diffX = WIDTH/2 + CURSOR_OFFSET - cursorPosition.x;
      diffY = HEIGHT/2 - cursorPosition.y;
    }
      
    cursorPosition.x += diffX * ROTATION_SPEED_APPROACH / 100;
    cursorPosition.y += diffY * ROTATION_SPEED_APPROACH / 100;

    rotationX = ( cursorPosition.x - WIDTH/2 ) * ROTATION_SPEED / 5000;
    rotationY = ( cursorPosition.y - HEIGHT/2 ) * ROTATION_SPEED / 5000;
    
    up = new Vector( 0, 1, 0 );
    cameraPosition = cameraPosition.arbitraryRotate( up, -rotationX );
    
    // Disable y rotation for now...
    // cross = up.cross( cameraPosition );
    // cameraPosition = cameraPosition.arbitraryRotate( cross, -rotationY );
  }
  
  int findClosestNicePointIndex()
  {
    Vector diff = nicePoints[0].sub( cameraPosition );
    float closestLength = diff.length();
    int index = 0;
    
    for ( int i=1; i<4; i++ )
    {
      diff = nicePoints[i].sub( cameraPosition );
      
      if ( diff.length() < closestLength )
      {
        closestLength = diff.length();
        index = i;
      }
    }
    
    return index;
  }
  
  Vector findClosestNicePoint
  ()
  {
    return nicePoints[ findClosestNicePointIndex() ];
  }
  
  void moveInAndOut
  ( float k )
  {
    cameraPosition = cameraPosition.mul( k );
    SNAP_RADIUS *= k;
    
    for ( int i=0; i<nicePoints.length; i++ )
    {
      nicePoints[i] = nicePoints[i].mul( k );
    }
  }
  
  void rotateRight
  ()
  {
    Vector up = new Vector( 0, 1, 0 );
    cameraPosition = cameraPosition.arbitraryRotate( up, radians( 85 ) );
  }
  
  void rotateLeft
  ()
  {
    Vector up = new Vector( 0, 1, 0 );
    cameraPosition = cameraPosition.arbitraryRotate( up, radians( -85 ) );
  }
}
