// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2008  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Apologies for the lack of comments... I never thought this would end up
// online. If you have any questions feel free to email me at:
// ali@arkitus.com
// ------------------------------------------------------------------------------

float    SCENE_RADIUS              = 200;
int      WIDTH                     = 800;
int      HEIGHT                    = 420;
int      DEPTH                     = 420;
float    BOX_SIZE                  = 7;
float    BOX_RADIUS                = 30;
float    GRID_SPACING              = -2;
float    ROTATION_SPEED_APPROACH   = 10;
float    ROTATION_SPEED            = 1;
float    CURSOR_OFFSET             = 0;
int      CHARACTER_WIDTH           = 8;
int      CHARACTER_HEIGHT          = 8;
float    FILL_RATE                 = 120;
float    CAMERA_DISTANCE           = 4;
float    CAMERA_FOV                = 3;
float    CAMERA_HEIGHT             = 0;
int      NUMBER_OF_BOXES           = 300;
float    SNAP_RADIUS               = 300;
float    SNAP_SPEED                = 0.1;

Scene          mainScene;
Camera         mainCamera;
Renderer       mainRenderer;
CharacterSet   characterSet;

void setup
() 
{  
  mainScene = new Scene();
  mainCamera = new Camera();
  mainRenderer = new Renderer();
  characterSet = new CharacterSet();
}

void draw
()
{
  mainScene.tick();
  mainCamera.tick();
  mainRenderer.draw( mainScene, mainCamera );
}

void keyPressed
()
{
  if ( keyCode == UP )
  {
    mainCamera.moveInAndOut( 0.95 );
  }
  else if ( keyCode == DOWN )
  {
    mainCamera.moveInAndOut( 1.05 );
  }
  else if ( keyCode == RIGHT )
  {
    mainCamera.rotateRight();
  }
  else if ( keyCode == LEFT )
  {
    mainCamera.rotateLeft();
  }
  else
  {
    mainScene.setCharacter( characterSet.getCharacter( key ), mainCamera );
  }
}

Vector findPositionOnSphere
( Box[] boxes )
{
  while ( true ) 
  {
    float x1 = random( -1, 1 );
    float x2 = random( -1, 1 );
        
    if ( x1 * x1 + x2 * x2 >= 1 )
    {
      continue;
    }
        
    float r = SCENE_RADIUS;
    float x = 2 * r * x1 * sqrt( 1 - x1 * x1 - x2 * x2 );
    float y = 2 * r * x2 * sqrt( 1 - x1 * x1 - x2 * x2 );
    float z = r * ( 1 - 2 * ( x1 * x1 + x2 * x2 ) );
    Vector position = new Vector( x, y, z );
        
    if ( isTooCloseToOtherBoxes( position, boxes ) )
    {
      continue;
    }
        
    return position;
  }
}
  
boolean isTooCloseToOtherBoxes
( Vector position,
  Box[] boxes)
{
  Vector diff;
   
  for ( int i=0; i<boxes.length; i++ )
  {
    if ( boxes[i] != null )
    {
      diff = boxes[i].position.sub( position );
        
      if ( diff.length2() <= BOX_RADIUS * BOX_RADIUS )
      {
        return true;
      }
    }
  }
    
  return false;
}
